@@ -1,6 +1,6 @@
 /*
 integrit - file integrity verification system
-Copyright (C) 2000, 2001 Ed L. Cashin
+Copyright (C) 2000, 2001, 2002 Ed L. Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	<string.h>
 #include	<fcntl.h>
 #include	<dirent.h>
+#include	<time.h>
 #include	<utime.h>
 #include	<sys/types.h>
 #include	<sys/stat.h>
@@ -46,16 +47,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	"elcerror.h"
 #include	"xstrdup.h"
 #include	"xstradd.h"
-#include	"options.h"
+#include	"integrit.h"
 #include	"rules.h"
 #include	"checkset.h"
 #include	"elcwft.h"
 #include	"eachfile.h"
-#include	"utilities.h"
 #include	"xml.h"
 #include	"elcerror_p.h"
 #include	"rules_p.h"
-#include	"utilities_p.h"
+#include	"hexprint_p.h"
 #include	"eachfile_p.h"
 #include	"xml_p.h"
 #include	"dbinfo.h"
@@ -64,6 +64,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	"leakfind.h"
 #endif
 #define		ALGO_NAME	"SHA-1"
+#define		ELC_TIMEBUFSIZ	16
 
 
 typedef struct fileinfo {	/* database file information */
@@ -87,10 +88,10 @@ static void fileinfo_init(fileinfo *inf, char *path, size_t pathlen)
     inf->pathlen	 = pathlen;
 }
 
-static unsigned long fileinfo_ruleset(options *opts, fileinfo *inf)
+static unsigned long fileinfo_ruleset(integrit_t *it, fileinfo *inf)
 {
     if (! inf->found_ruleset) {
-      inf->ruleset		 = rules_for_path(opts, inf->path);
+      inf->ruleset		 = rules_for_path(it, inf->path);
       inf->found_ruleset	 = 1;
     }
 
@@ -111,7 +112,7 @@ static unsigned long fileinfo_ruleset(options *opts, fileinfo *inf)
  * Thanks to lumpy <lumpy@9mm.com> for pointing out the race condition.
  *
  */
-inline static int do_checksum(options *opts, fileinfo *inf)
+inline static int do_checksum(integrit_t *it, fileinfo *inf)
 {
     SHA1_CONTEXT	context;
     char	buf[BUFSIZ];
@@ -155,7 +156,7 @@ inline static int do_checksum(options *opts, fileinfo *inf)
     memcpy(dbinf->sum, context.buf, SHA1_DIGEST_LENGTH);
 
     /* reset access time on request */
-    if (fileinfo_ruleset(opts, inf) & RULE_RESET_ATIME) {
+    if (fileinfo_ruleset(it, inf) & RULE_RESET_ATIME) {
 #ifdef	DEBUG
       fprintf(stderr, "debug (%s): resetting atime for file (%s)\n",
 	      __FUNCTION__, inf->path);
@@ -193,19 +194,18 @@ static void show_diff_long(FILE *out, char T, const char *type,
 static void show_diff_time(FILE *out, char T, const char *type,
 				 time_t old, time_t new)
 {
-    const int	buf_max	 = 16;	/* 20001212-010101 + null */
-    char	buf[buf_max];
+    char	buf[ELC_TIMEBUFSIZ];
 
     putc(T, out);
     putc('(', out);
 
-    if (! strftime(buf, buf_max, "%Y%m%d-%H%M%S", localtime(&old)) )
+    if (! strftime(buf, ELC_TIMEBUFSIZ, "%Y%m%d-%H%M%S", localtime(&old)) )
       DIE("strftime");
     fputs(buf, out);
 
     putc(':', out);
 
-    if (! strftime(buf, buf_max, "%Y%m%d-%H%M%S", localtime(&new)) )
+    if (! strftime(buf, ELC_TIMEBUFSIZ, "%Y%m%d-%H%M%S", localtime(&new)) )
       DIE("strftime");
     fputs(buf, out);
 
@@ -247,7 +247,7 @@ static void show_diff_perms(FILE *out, mode_t old, mode_t new)
       fprintf(out, "%o) ", (unsigned int) new & PERM_MASK);
 }
 
-static void show_diffs(options *opts,
+static void show_diffs(integrit_t *it,
 		       const char *path, unsigned long diffs,
 		       const struct stat *sa, const struct stat *sb)
 {
@@ -259,9 +259,11 @@ static void show_diffs(options *opts,
     void (*sho_oct)(FILE *out, char T, const char *type,
 		     unsigned long old, unsigned long new);
 
-    if (opts->output == OUTPUT_XML) {
+    if (it->output == OUTPUT_XML) {
       sho_long	 = show_diff_xml_long;
-      sho_time	 = show_diff_xml_long;
+      /* this cast surpresses warnings on some systems */
+      sho_time
+	= (void (*)(FILE *, char, const char *, time_t, time_t)) show_diff_xml_long;
       sho_oct	 = show_diff_xml_octal;
       XML_CHANGE_START_PRINT(stdout, "stat", path);
     } else {
@@ -274,7 +276,7 @@ static void show_diffs(options *opts,
     if (diffs & RULE_INODE)
       sho_long(stdout, 'i', "inode", sa->st_ino, sb->st_ino);
     if (diffs & RULE_PERMS) {
-      if (opts->output == OUTPUT_XML) {
+      if (it->output == OUTPUT_XML) {
 	sho_oct(stdout, 'p', "permissions",
 		 sa->st_mode & PERM_MASK, sb->st_mode & PERM_MASK);
       } else {
@@ -296,19 +298,19 @@ static void show_diffs(options *opts,
     if (diffs & RULE_CTIME)
       sho_time(stdout, 'c', "change_time", sa->st_ctime, sb->st_ctime);
 
-    if (opts->output == OUTPUT_XML)
+    if (it->output == OUTPUT_XML)
       XML_END_PRINT(stdout, "change");
 
     putc('\n', stdout);
 }
 
-static void report_stat_differences(options *opts,
+static void report_stat_differences(integrit_t *it,
 				    fileinfo *currinf, dbinfo *old)
 {
     struct stat		*sa	 = &old->stat;
     struct stat		*sb	 = &currinf->dbinf.stat;
     const char *path		 = currinf->path;
-    unsigned long	flags	 = fileinfo_ruleset(opts, currinf);
+    unsigned long	flags	 = fileinfo_ruleset(it, currinf);
     unsigned long	diffs	 = 0;
     
     if ((flags & RULE_INODE)
@@ -339,15 +341,19 @@ static void report_stat_differences(options *opts,
 	&& (sa->st_ctime != sb->st_ctime))
       diffs	 |= RULE_CTIME;
 
-    if (diffs)
-      show_diffs(opts, path, diffs, sa, sb);
+    if (diffs) {
+      it->exit_status |= INTEGRIT_EXIT_CHANGES;
+      show_diffs(it, path, diffs, sa, sb);
+    }
 }
 
 /* report that a file has changed from a non-regular file
  * to a regular file */
-static void report_2regfile(options *opts, const char *path)
+static void report_2regfile(integrit_t *it, const char *path)
 {
-    switch (opts->output) {
+    it->exit_status |= INTEGRIT_EXIT_CHANGES;
+
+    switch (it->output) {
       case OUTPUT_XML:
 	XML_CHANGE_START_PRINT(stdout, "filetype", path);
 	XML_ELEMENT_PRINT(stdout, "old", "not regular file");
@@ -364,11 +370,13 @@ static void report_2regfile(options *opts, const char *path)
 }
 
 /* report that the checksum has changed. */
-static void report_sumchange(options *opts, const char *path,
+static void report_sumchange(integrit_t *it, const char *path,
 			     const unsigned char *old, size_t oldsiz,
 			     const unsigned char *new, size_t newsiz)
 {
-    switch (opts->output) {
+    it->exit_status |= INTEGRIT_EXIT_CHANGES;
+
+    switch (it->output) {
       case OUTPUT_LINES:
 	printf("changed: %s   s(", path);
 	hexprint(stdout, old, oldsiz);
@@ -392,13 +400,13 @@ static void report_sumchange(options *opts, const char *path,
     }
 }
 
-static void report_differences(options *opts, fileinfo *currinf)
+static void report_differences(integrit_t *it, fileinfo *currinf)
 {
     dbinfo		old;
-    struct cdb		*knowndb	 = &opts->knowndb;
+    struct cdb		*knowndb	 = &it->knowndb;
     size_t		knownsiz	 = cdb_datalen(knowndb);
     char		*path		 = currinf->path;
-    unsigned long	flags		 = fileinfo_ruleset(opts, currinf);
+    unsigned long	flags		 = fileinfo_ruleset(it, currinf);
     
     if (knownsiz != sizeof(old)
 	&& knownsiz != sizeof(old.stat))
@@ -410,18 +418,18 @@ static void report_differences(options *opts, fileinfo *currinf)
 
     if (S_ISREG(currinf->dbinf.stat.st_mode)) {	/* if it's a regular file */
       if (knownsiz != sizeof(dbinfo)) /* maybe known has no checksum */
-	report_2regfile(opts, path);
+	report_2regfile(it, path);
       else if (flags & RULE_SUM) {
 	if (! currinf->did_sum)
-	  currinf->did_sum	 = do_checksum(opts, currinf);
+	  currinf->did_sum	 = do_checksum(it, currinf);
 
 	if (currinf->did_sum
 	    && memcmp(currinf->dbinf.sum, old.sum, sizeof(old.sum)))
-	  report_sumchange(opts, path, old.sum, sizeof(old.sum),
+	  report_sumchange(it, path, old.sum, sizeof(old.sum),
 			   currinf->dbinf.sum, sizeof(currinf->dbinf.sum));
       }
     }
-    report_stat_differences(opts, currinf, &old);
+    report_stat_differences(it, currinf, &old);
 }
 
 static void show_xml_long(FILE *out, const char *type, long val)
@@ -438,13 +446,13 @@ static void show_xml_octal(FILE *out, const char *type, unsigned long val)
 }
 
 
-/* similar to aux/show.c: show_entry */
-static void report_newfile(options *opts, fileinfo *inf)
+/* similar to utils/show.c: show_entry */
+static void report_newfile(integrit_t *it, fileinfo *inf)
 {
     char	*path	 = inf->path;
     struct stat	*s	 = &inf->dbinf.stat;
 
-    switch (opts->output) {
+    switch (it->output) {
       case OUTPUT_XML:
 	XML_CHANGE_START_PRINT(stdout, "newfile", path);
 	show_xml_octal(stdout, "permissions", s->st_mode & PERM_MASK);
@@ -476,24 +484,27 @@ static void report_newfile(options *opts, fileinfo *inf)
     putc('\n', stdout);
 }    
 
-inline static void do_check(options *opts, fileinfo *inf)
+inline static void do_check(integrit_t *it, fileinfo *inf)
 {
-    struct cdb	*db	 = &opts->knowndb;
+    struct cdb	*db	 = &it->knowndb;
     int		err;
 
     if ( (err = cdb_find(db, inf->path, inf->pathlen)) == -1)
       die(__FUNCTION__,
 	  "Error: looking up file (%s) in known database (%s): %s",
-	  inf->path, opts->knowndbname, strerror(errno));
-    else if (!err)
-      report_newfile(opts, inf);
-    else
-      report_differences(opts, inf);
+	  inf->path, it->knowndbname, strerror(errno));
+
+    if (!err) {
+      it->exit_status |= INTEGRIT_EXIT_CHANGES;
+      report_newfile(it, inf);
+    } else {
+      report_differences(it, inf);
+    }
 }
 
-inline static void do_update(options *opts, fileinfo *inf)
+inline static void do_update(integrit_t *it, fileinfo *inf)
 {
-    struct cdb_make	*db	 = &opts->currdb;
+    struct cdb_make	*db	 = &it->currdb;
     dbinfo		*dbinf	 = &inf->dbinf;
     size_t		datasiz;
     
@@ -501,7 +512,7 @@ inline static void do_update(options *opts, fileinfo *inf)
       /* BUGGISH: don't test inf->did_sum, since this function's called
        *          before do_check (very minorly buggish :)
        */
-      inf->did_sum	 = do_checksum(opts, inf);
+      inf->did_sum	 = do_checksum(it, inf);
 
     datasiz	 = inf->did_sum ? sizeof(*dbinf) : sizeof(dbinf->stat);
 
@@ -511,7 +522,7 @@ inline static void do_update(options *opts, fileinfo *inf)
 
 wft_ret_t process_file(const char *path, const struct stat *sb, void *data)
 {
-    options	*opts		 = (options *) data;
+    integrit_t	*it		 = (integrit_t *) data;
     fileinfo	inf;		/* initialize before use */
     checkset	*cset;
     size_t	plen		 = strlen(path);
@@ -549,7 +560,7 @@ wft_ret_t process_file(const char *path, const struct stat *sb, void *data)
     /* initialize inf now that we've settled on a file path */
     fileinfo_init(&inf, p, plen);
 
-    cset	 = hashtbl_lookup(opts->ruleset, p, plen);
+    cset	 = hashtbl_lookup(it->ruleset, p, plen);
 
     /* see whether checkset includes boolean to ignore the file */
     if (cset && (cset->flags & RULE_IGNORE)) { 
@@ -562,11 +573,11 @@ wft_ret_t process_file(const char *path, const struct stat *sb, void *data)
 
     memcpy(&inf.dbinf.stat, sb, sizeof(inf.dbinf.stat));
 
-    if (opts->do_update)
-      do_update(opts, &inf);
+    if (it->do_update)
+      do_update(it, &inf);
 
-    if (opts->do_check)
-      do_check(opts, &inf);
+    if (it->do_check)
+      do_check(it, &inf);
 
     free(pathcopy);
 
